#
#
# $Id$
#
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License, version 2.
#
# Infobrowser support for the BBC Whats On  items
#
package Plugins::InfoBrowserAddons::RecentlyPlayedSomaFM::RecentlyPlayedSomaFMParser;

# Parser for SomaFM's recently played pages

use strict;

use Slim::Utils::Log;

use HTML::PullParser;
use HTML::Entities;
use Date::Parse qw(str2time);

my $log = logger('plugin.infobrowser');


sub getDisplayName { 'Recently Played SomaFM ' };

sub getnexttag
{
  my $t;
  my $p         = shift;
  my $searchtag = shift;
  my $exittag   = shift ;

  for (;;) {
     $t = $p->get_token();
     return undef unless (defined($t));
     return $t if ( $t->[1] eq $searchtag) ;
     return $t if ( defined ($exittag) && $t->[1] eq $exittag) ;
  }
}


sub cleanup
{
  my $str = shift;
  return undef unless defined ($str); 
  
  $str =~ s/\n/ /g;  # change LF to space
  $str =~ s/\r//g;   # Get rid of CR if any.
  $str =~ s/<br>//g; # Get rid of HTML <br> if any.
  $str =~ s/<br \/>//g; # Get rid of HTML <br> if any.
  $str =~ s/<BR>//g; # Get rid of HTML <BR> if any.
  $str =~ s/<B>//g; # Get rid of HTML <B> if any.
  $str =~ s/<\/B>//g; # Get rid of HTML </B> if any.
  $str =~ s/&nbsp;/ /g; # replace HTML &nbsp if any.

# strip whitespace from beginning and end

  $str =~ s/^\s*//; 
  $str =~ s/\s*$//; 
  

  return $str;
}


sub parse
{

	my $class  = shift;
	my $http   = shift;

	my $params = $http->params('params');
	my $url    = $params->{'url'};
	my $title  = $params->{'text'};
	my @list;
	my $token;
	my $nowtime = 0;
	my $currenttime;

	my ( $playedtime , $playedsong,  $playedartist, $playedalbum);

	my $p = HTML::PullParser->new(api_version => 3, doc => ${$http->contentRef},
		      	    	  start => 'event, tag, attr,  skipped_text',
			    	  end   => 'event, tag, dtext, skipped_text',
			          report_tags => [qw ( td tr table div a)]);

    while($token = getnexttag($p,"div")) {

        next unless defined($token->[2]->{id});
        last if $token->[2]->{id} eq "content";
    }
    $log->info(" Found start of content ") if (defined($token));

    while($token = getnexttag($p,"div")) {
        next unless defined($token->[2]->{id});
        last if $token->[2]->{id} eq "playinc";
    }
    $log->info(" Found start of played lines") if (defined($token));
	# get first table with Titles
	$token = getnexttag($p,"table");
	$token = getnexttag($p,"tr");
	$token = getnexttag($p,"tr");

#<!-- line 1 -->
#<tr>
#     <td>16:42:44&nbsp; (Now) </td>
#     <td>
#         <a  target="_blank" href="http://www.justmusic.co.uk" title="Go to Jon Hopkins&#146;s site" >
#         Jon Hopkins
#         </a>
#     </td>
#     <td>
#         Sleepwalker
#     </td>
#     <td>
#         <a target="_blank" href="/buy/multibuy.cgi?mode=amazon&amp;title=Sleepwalker&amp;album=Contact%20Note&amp;artist=Jon%20Hopkins" title="Search Amazon.com for Contact Note" >
#         Contact Note
#         </a>
#     </td>
#<td></td>
#</tr>
#
#<tr>
#    <td>16:25:52</td>
#    <td colspan="4"><span class="dim">Break / Station ID</span></td>
#</tr>
	while ($token = getnexttag($p,"tr","/table")) {
		last if ($token->[1] eq "/table");

		$token = getnexttag($p,"td");
		if (defined($token->[2]->{colspan})) {
			$token = getnexttag($p,"/tr");
			next;
		}

		$token = getnexttag($p,"/td");
		$playedtime = cleanup ($token->[3] ) ;
		if ($nowtime == 0) {
			$nowtime = str2time($playedtime);
		}
		$currenttime = str2time($playedtime);
			
		$token = getnexttag($p,"td");
		if (defined($token->[2]->{colspan})) {
			$token = getnexttag($p,"/td");
		} else { 
			$token = getnexttag($p,"/a");
			$playedartist = cleanup ($token->[3] ) ;
			$token = getnexttag($p,"/td");

			$token = getnexttag($p,"/td");
			$playedsong = cleanup ($token->[3] ) ;
			$token = getnexttag($p,"/a");
			$playedalbum = cleanup ($token->[3] ) ;
			my $ago = ($nowtime-$currenttime);
			if ($ago < 0 ) { 
				$ago = $ago + (24*60*60);
			};
		        $log->debug( "date=$playedtime secs ago= $ago artist=$playedartist song=$playedsong album=$playedalbum");

		         push @list,{	'name'   =>  "$playedsong by $playedartist",
					'description'   =>  "Song: $playedsong  \nArtist : $playedartist  \nAlbum: $playedalbum  \nPlayed at: $playedtime"
		 	};


		} ;
		$token = getnexttag($p,"/tr");
	}		

return {
		'type'  => 'opml',
		'name' => "RecentlyPlayedSomaFM - parsed",
		'description' => "Recently played on $title ",
		'items' => [ @list],
	};

}

# Local Variables:
# tab-width:4
# indent-tabs-mode:t
# End:

1;
